/*
Copyright 2010-2012 Michael Braiman

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package ui.auto.core.context;

import org.openqa.selenium.Capabilities;
import org.openqa.selenium.Platform;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebDriver.Window;
import org.openqa.selenium.remote.DesiredCapabilities;

import ui.auto.core.data.DataSet;
import ui.auto.core.data.TestDataDefinition;
import ui.auto.core.pageobjects.PageObject;

public class TestContext {
	private WebDriver driver;
	private String browser;
	private int ajaxTimeOut=10; //in seconds
	private int waitForUrlTimeOut=10000; //in milliseconds 
	private PageObject currentPageObject;
	private TestDataDefinition testDataDefinition;
	private DataSet dataSet;
	private String serverUrl;
	private WebDriverProvider driverProvider;

	protected TestContext(TestDataDefinition testDataDefinition, DataSet dataSet,boolean createLazily) {
		this.testDataDefinition=testDataDefinition;
		this.serverUrl=testDataDefinition.getServerUrl();
		this.dataSet=dataSet;
		if (dataSet.getBrowser()==null){
			throw new RuntimeException("Browser was not provided in DataSet, please add browser attribute to the data-set element of DataDefinition file. " +
					"\nOtherwise provide your implementation to creatte WebDriver using WebDriverProvider interface");
		} 
		this.browser=dataSet.getBrowser().name();
		if (!createLazily) init();
	}
	
	protected TestContext(TestDataDefinition testDataDefinition, DataSet dataSet,WebDriverProvider webDriverProvider,boolean createLazily) {
		this.testDataDefinition=testDataDefinition;
		this.serverUrl=testDataDefinition.getServerUrl();
		this.dataSet=dataSet;
		browser="custom";
		driverProvider=webDriverProvider;
		if (!createLazily) init();
	}
	
	public TestContext(BrowserDriverType browser,String serverUrl,String urlPrefix,String user,String password) {
		testDataDefinition=new TestDataDefinition(serverUrl,urlPrefix,user,password);
		this.browser=browser.name();
		this.serverUrl=serverUrl;
		driver=browser.getNewDriver(null);
		loadServerUrl();
	}
	
	public TestContext(WebDriver driver,String serverUrl){
		testDataDefinition=new TestDataDefinition(serverUrl,"","","");
		this.driver=driver;
		browser="unknown";
		this.serverUrl=serverUrl;
		loadServerUrl();
	}
	
	private void loadServerUrl(){
		if (serverUrl!=null && !serverUrl.trim().isEmpty())
			driver.get(serverUrl);
	}
	
	public String getServerUrl(){
		if (testDataDefinition!=null){
			return testDataDefinition.getServerUrl();
		}
		return serverUrl;
	}
	
	private void init(){
		if (driver==null) {
			if (driverProvider!=null) {
				driver=driverProvider.provide();
			} else {
				String hubUrl=dataSet.getHubUrl();
				Platform platform= dataSet.getPlatform();
				String version=dataSet.getVersion();
				BrowserDriverType browser=dataSet.getBrowser();
				Capabilities extraCapabilities=null;
				if  (dataSet.getCapabilities()!=null) 
					extraCapabilities=new DesiredCapabilities(dataSet.getCapabilities());
				if (hubUrl!=null && !hubUrl.trim().equals("")){
					DesiredCapabilities capabilities=new DesiredCapabilities(browser.getDesiredBrowserName(),version,platform);
					if (extraCapabilities!=null) capabilities.merge(extraCapabilities);
					driver=browser.getNewRemoteDriver(hubUrl,capabilities);
				} else {
					driver=browser.getNewDriver(extraCapabilities);
				}
			}
			loadServerUrl();
		}		
	}
	
	
	public TestDataDefinition getTestDataDefinition() {
		return testDataDefinition;
	}

	public DataSet getDataSet() {
		return dataSet;
	}

	public int getWaitForUrlTimeOut() {
		return waitForUrlTimeOut;
	}

	public void setWaitForUrlTimeOut(int waitForUrlTimeOut) {
		this.waitForUrlTimeOut = waitForUrlTimeOut;
	}

	public PageObject getCurrentPageObject() {
		return currentPageObject;
	}

	public void setCurrentPageObject(PageObject currentPageObject) {
		this.currentPageObject = currentPageObject;
	}

	public String getBrowser() {
		return browser;
	}

	public WebDriver getDriver() {
		init();
		return driver;
	}
	
	public Window getWindow(){
		init();
		return driver.manage().window();
	}

	public void close(){
		if (driver!=null) this.driver.close();
	}
	
	public void quit(){
		if (driver!=null) this.driver.quit();
	}
	
	@Override
	public String toString(){
		return browser + " : " + dataSet; 
	}

	public int getAjaxTimeOut() {
		return ajaxTimeOut;
	}

	public void setAjaxTimeOut(int ajaxTimeOut) {
		this.ajaxTimeOut = ajaxTimeOut;
	}

	
}
	
