/*
Copyright 2010-2012 Michael Braiman

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package ui.auto.core.pageobjects;

import java.lang.reflect.Field;

import org.openqa.selenium.support.PageFactory;
import org.openqa.selenium.support.pagefactory.DefaultElementLocatorFactory;

import ui.auto.core.context.TestContext;
import ui.auto.core.data.DataPersistence;
import ui.auto.core.data.generators.DataSetGenerator;
import ui.auto.core.utils.Utils;

import com.thoughtworks.xstream.annotations.XStreamOmitField;


public class PageObject extends DataPersistence{
	@XStreamOmitField
	protected TestContext context=null;
	@XStreamOmitField
	private String currentUrl;
	@XStreamOmitField
	private boolean ajaxIsUsed;;
	
	
	protected PageObject(TestContext context){
		initPage(context);
	}
	
	protected PageObject(TestContext context,String expectedUrl){
		initPage(context,expectedUrl);
	}
	
	protected PageObject(TestContext context,boolean ajaxIsUsed){
		initPage(context,ajaxIsUsed);
	}
	
	protected PageObject(TestContext context,String expectedUrl,boolean ajaxIsUsed){
		initPage(context,expectedUrl,ajaxIsUsed);
	}
	
	
	public void  initPage(TestContext context,boolean ajaxIsUsed){
		this.context=context;
		this.ajaxIsUsed=ajaxIsUsed;
		context.setCurrentPageObject(this);
		currentUrl=context.getDriver().getCurrentUrl();
		if (ajaxIsUsed){
			AjaxVisibleElementLocatorFactory ajaxVisibleElementLocatorFactory=new AjaxVisibleElementLocatorFactory(context.getDriver(),context.getAjaxTimeOut());
			PageFactory.initElements(new ComponentFieldDecorator(ajaxVisibleElementLocatorFactory,this),this);
		}else {
			DefaultElementLocatorFactory defLocFactory=new DefaultElementLocatorFactory(context.getDriver());
			PageFactory.initElements(new ComponentFieldDecorator(defLocFactory, this),this);
		}
	}
	
	public void  initPage(TestContext context){
		initPage(context,true);
	}
	
	public void initPage(TestContext context,String expectedUrl){
		initPage(context,expectedUrl,true);
	}
	
	private void initPage(TestContext context2, String expectedUrl,boolean ajaxIsUsed) {
		initPage(context,ajaxIsUsed);
		if (expectedUrl!=null) waitForUrl(expectedUrl);
	}

	public TestContext getTestContext(){
		return context;
	}
	
	public void waitForUrl(String expectedUrl){
		long endTime=System.currentTimeMillis() + context.getWaitForUrlTimeOut();
		while (System.currentTimeMillis()<endTime) {
			if (context.getDriver().getCurrentUrl().contains(expectedUrl)) {
				currentUrl=context.getDriver().getCurrentUrl();
				return;
			}
			Utils.Sleep(100);
		} 
		throw new RuntimeException("Expected url:" + expectedUrl + " is not displayed!");	
	}
	
	public boolean waitForUrlToChange(){
		long endTime=System.currentTimeMillis() + context.getWaitForUrlTimeOut();
		boolean exitWhile=false;
		while (System.currentTimeMillis()<endTime && !exitWhile) {
			if (!context.getDriver().getCurrentUrl().contains(currentUrl)) exitWhile=true;
			Utils.Sleep(100);
		} 
		return exitWhile;
	}
	
	protected void setElementValue(WebComponent component){
		setElementValue(component,true);
	}
	
	protected void setElementValue(WebComponent component,String value){
		setElementValue(component,value,true);
	}
	
	protected void setElementValue(WebComponent component,boolean validate){
		if (component.getData()!=null && !component.getData().isEmpty()) {
			for (int i=0;i<3;i++){
				component.setValue();
				if (!validate) return;
				if (component.getValue().equals(component.getData())) return;
			}
			throw new RuntimeException("Can't set element '" + component.getClass().getSimpleName() + "' to value: " + component.getData());
		}
	}
	
	protected void setElementValue(WebComponent component,String value,boolean validate){
		String realValue=null;
		if (component.getData()!=null && !component.getData().isEmpty()) 
			realValue=component.getData();
		component.setData(value);
		setElementValue(component,validate);
		component.setData(realValue);
	}
	
	protected void enumerateFields(FieldEnumerationAction action) {
		for (Field field:this.getClass().getDeclaredFields()){
			if (!field.isAnnotationPresent(XStreamOmitField.class) && !field.isAnnotationPresent(SkipAutoFill.class)) {
				if (WebComponent.class.isAssignableFrom(field.getType())){
					field.setAccessible(true);
					WebComponent component=null;
					try {
						component = (WebComponent) field.get(this);
					} catch (IllegalAccessException e) {
						throw new RuntimeException(e);
					}
					action.doAction(component,field);
				}
			}
		}
	}
	
	protected void autoFillPage(){
		autoFillPage(true);
	}
	
	protected void autoFillPage(final boolean validate){
		if (context==null) throw new RuntimeException("PageObject is not initialized, invoke initPage method!");
		enumerateFields(new FieldEnumerationAction() {	
			
			@Override
			public void doAction(WebComponent webComponent,Field field) {
				setElementValue(webComponent,validate);
			}
		});
	
	}
	
	protected void autoAssertPage(){
		if (context==null) throw new RuntimeException("PageObject is not initialized, invoke initPage method!");
		enumerateFields(new FieldEnumerationAction() {
			
			@Override
			public void doAction(WebComponent webComponent,Field field) {
				webComponent.validate();
			}
		});
	}
	
	public void generateData(){
		String generatedFilePath=DataSetGenerator.getInstance().generateDataSet(this);
		if (context!=null){
			initPage(context,ajaxIsUsed);
			if (context.getDataSet()!=null)
				context.getDataSet().addFilePath(generatedFilePath);
		}
	}
	
	public void initData(PageObject pageObject){
		TestContext currentContext=context;
		deepCopy(pageObject,this);
		initPage(currentContext);
	}
	
}
