/*
Copyright 2010-2012 Michael Braiman

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package ui.auto.core.pageobjects;

import java.util.List;
import java.util.NoSuchElementException;


import org.openqa.selenium.By;
import org.openqa.selenium.Dimension;
import org.openqa.selenium.ElementNotVisibleException;
import org.openqa.selenium.InvalidElementStateException;
import org.openqa.selenium.Point;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.internal.Locatable;
import org.openqa.selenium.support.ui.ExpectedCondition;
import org.openqa.selenium.support.ui.WebDriverWait;

import com.thoughtworks.xstream.annotations.XStreamAlias;

import ui.auto.core.data.WebComponentData;
import ui.auto.core.utils.Utils;

@XStreamAlias("WebComponent")
public abstract class WebComponent implements WebElement,WebComponentData {
	protected WebElement element;
	protected String data;
	
	public WebComponent(String data){
		this.setData(data);
	}
	
	public WebComponent(WebElement element){
		this.element=element;
		init();
	}
	
	//Override this class for additional initialisation of component
	protected abstract void init();
	
	@Override
	public void click() {
		element.click();
	}
	@Override
	public void submit() {
		element.submit();	
	}
	@Override
	public void sendKeys(CharSequence... keysToSend) {
		element.sendKeys(keysToSend);
	}
	@Override
	public void clear() {
		element.clear();	
	}
	@Override
	public String getTagName() {
		return element.getTagName();
	}
	@Override
	public String getAttribute(String name) {
		return element.getAttribute(name);
	}
	@Override
	public boolean isSelected() {
		return element.isSelected();
	}
	@Override
	public boolean isEnabled() {
		return element.isEnabled();
	}
	@Override
	public String getText() {
		return element.getText();
	}
	@Override
	public List<WebElement> findElements(By by) {
		return element.findElements(by);
	}
	@Override
	public WebElement findElement(By by) {
		return element.findElement(by);
	}
	@Override
	public boolean isDisplayed() {
		return element.isDisplayed();
	}
	@Override
	public Point getLocation() {
		return element.getLocation();
	}
	@Override
	public Dimension getSize() {
		return element.getSize();
	}
	@Override
	public String getCssValue(String propertyName) {
		return element.getCssValue(propertyName);
	}
	@Override
	public String getData() {
		return data;
	}
	@Override
	public void setData(String data) {
		this.data = data;
	}	
	
	public WebElement getWrappedWebElement(){
		return element;
	}
	
	public void scrollIntoView(){
		((Locatable) element).getLocationOnScreenOnceScrolledIntoView();
	}

	
	public abstract void setValue();//element.clear();element.sendKeys(data);
	public abstract String getValue();//return element.getAttribute("value");
	public abstract void validate();//Assert.assertEquals(element.getAttribute("value"),data);
	
	protected void verifyTag(String tagName){
		String tag=element.getTagName();
		if (!tagName.equals(tag))
			throw new RuntimeException ("ERROR: Expected tag is " + tagName + " but was " + tag); 
	}
	
	protected void verifyType(String typeName){
		String type=element.getAttribute("type");
		if (!typeName.equals(type))
			throw new RuntimeException ("ERROR: Expected type is " + typeName + " but was " + type); 
	}
	
	protected WebElement waitForElement(final By by,long timeOut){
		WebDriver driver=Utils.getDriverFromElement(element);
		WebElement ajaxElement = new WebDriverWait(driver,timeOut).until(new ExpectedCondition<WebElement>() {
			@Override
			public WebElement apply(WebDriver d) {
				return d.findElement(by);
			}
		});
		return ajaxElement;
	}
	
	protected WebElement waitForVisibleElement(final By by,long timeOut){
		WebDriver driver=Utils.getDriverFromElement(element);
		WebDriverWait wwait=new WebDriverWait(driver,timeOut);
		wwait.ignoring(NoSuchElementException.class,ElementNotVisibleException.class);
		WebElement ajaxElement =wwait.until(new ExpectedCondition<WebElement>() {
			@Override
			public WebElement apply(WebDriver d) {
				WebElement el=d.findElement(by);
				if (el.isDisplayed()) return el;
				return null;
			}
		});
		return ajaxElement;
	}
	
	protected WebElement waitForVisibleEnabledElement(final By by,long timeOut){
		WebDriver driver=Utils.getDriverFromElement(element);
		WebDriverWait wwait=new WebDriverWait(driver,timeOut);
		wwait.ignoring(NoSuchElementException.class,ElementNotVisibleException.class);    
		wwait.ignoring(InvalidElementStateException.class);
		WebElement ajaxElement = wwait.until(new ExpectedCondition<WebElement>() {
			@Override
			public WebElement apply(WebDriver d) {
				WebElement el=d.findElement(by);
				if (el.isDisplayed() && el.isEnabled()) return el;
				return null;
			}
		});
		return ajaxElement;
	}
	
}
